/* CompSys 301

   Beta FETish

   Eugene Kin Chee Yip
   3849421

   08 January 2008
*/

/* C code to programme a custom built robot to follow a black line
 * against a white surface.  Hardware includes two motor driver inputs
 * (H-Bridges), three light sensor inputs, two IPT sensor inputs, one 
 * low-voltage IPT power supply input, one light-on output, five debug
 * outputs and one data output.  There is also one ISP header for
 * programmability.
 * 
 * System clock runs at 8 MHz
 * Prescaler of 8 gives timer frequency of 1 MHz
 * 8 bit resolution gives timer overflow frequency of 4.10 KHz
 * 
 * Robot can enter 5 modes chosen by a 4 way DIP switch mounted on robot
 * PCB.  Note that system mode is different to user mode (defined by datasheet).
 *
 *    DIP switch | Mode | Description
 *    1 2 3 4    |      |
 *   ---------------------------------
 *    0 0 0 0      0      Open loop
 *    1 0 0 0      1      Not used
 *    0 1 0 0      2      Not used
 *    0 0 1 0      3      Closed loop efficiency
 *    0 0 0 1      4      Closed loop acceleration/braking
 * 
 * Debugging outputs:  Pin B6 & B7 and Pin D0, D1 & D2
 * 
 */


// AVR include files
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/power.h>
#include <avr/portpins.h>


// Used for setting PWM duty cycles
#define SLOW	0xD0
#define MEDIUM	0xA0
#define FAST	0x80
#define MAX		0x00
#define OFF		0xFF


// Timing interval for acceleration / braking mode
#define	INTERVAL	1023

// Timeout for line detection and decoding
#define TIMEOUT		0x6000

// Period for one note


// Function prototypes
inline void startUp(void);
inline void initialise(void);
void movement(unsigned char);
void dipSwitchADC(void);
void iptADC(void);

inline void timerReset(void);
inline void timerStart(void);
inline void timerStop(void);
inline unsigned char timerHasTimeOut(void);

void gapInterpretation(void);
void gapSense(unsigned char);

void soundGenerator(void);
void playNote(unsigned long int, unsigned int);
void playRest(unsigned long int beats);
void playScale(void);
void playKookaburra(void);
void playCanon(void);
void playAir(void);

// Function pointers
void (*ADCroutine)(void) = 0;


/* Encodes motor direction
 * _F = Full power
 * _M = Medium power
 */
typedef enum { 	LEFT = 0, LEFT_M,
				RIGHT, RIGHT_M,
				MIDDLE,
				IDLE,
				REVERSE,
				BRAKE,
				INTERSECTION
			} direction;

/* Encodes intersection direction
 */
typedef enum {	LEFT_TURN = 1,
				RIGHT_TURN,
				STRAIGHT,
				IPT,
				END
			} markerDirection;

/* Encodes state of line decoding
*/
typedef enum {	LINE = 0,
				LINESEGMENT,
				GAP,
				FIRSTGAP,
				SECONDGAP,
				MARKER,
				TRACK,
				DEBOUNCING
			} gapState;

/* Encodes musical periods
 */
typedef enum {	SIXTEENTH_NOTE = 30000,
				EIGHTH_NOTE = 60000,
				QUARTER_NOTE = 120000,
				HALF_NOTE = 240000,
				WHOLE_NOTE = 480000,
				SIXTEENTH_REST = 30000,
				EIGHTH_REST = 60000,
				QUARTER_REST = 120000,
				HALF_REST = 240000,
				WHOLE_REST = 480000
			} period;
			
/* Encodes musical notes
 */
typedef enum {	B3			= 350,
				A3_SHARP	= 371,
				A3			= 393,
				G3_SHARP	= 418,
				G3			= 443,
				F3_SHARP	= 470,
				F3			= 498,
				E3			= 525,
				D3_SHARP	= 558,
				D3			= 592,
				C3_SHARP	= 627,
				C3			= 663,
				
				B4 			= 705,
				A4_SHARP	= 747,
				A4			= 789,
				G4_SHARP	= 837,
				G4			= 885,
				F4_SHARP	= 940,
				F4			= 996,
				E4			= 1055,
				D4_SHARP	= 1120,
				D4			= 1186,
				C4_SHARP	= 1258,
				C4			= 1330,
				
				B5 			= 1410,
				A5_SHARP	= 1492,
				A5			= 1575,
				G5_SHARP	= 1672,
				G5			= 1770,
				F5_SHARP	= 1880,
				F5			= 1990,
				E5			= 2105,
				D5_SHARP	= 2232,
				D5			= 2360,
				C5_SHARP	= 2500,
				C5			= 2650,
			} note;

/* Two arrays/look up tables for determining the next state of the robot.
 *
 * Bit pattern: Right, Middle, Left sensor values
 * Bit patterns correspond to index into array.
 *                                                  Bit pattern:                          */
const unsigned char nextMotorDir [8] = {	MIDDLE,			// 000		Gap in track
											LEFT,			// 001
											MIDDLE,			// 010
											LEFT_M,			// 011
											RIGHT,			// 100
											INTERSECTION,	// 101		Assume intersection
											RIGHT_M,		// 110
											INTERSECTION	// 111		Assume intersection
										};


unsigned char counter;							// Counter for sensor period.
volatile unsigned char sensor, sensorSnapShot;	// Current values.
unsigned long int delay, timerTicks;			// Variable for counting in an empty for-loop, Number of clock ticks.

unsigned char motorDir, mode, isTiming;			// Robot direction, robot mode, enabling timing.
unsigned long int marker[4] = {0, 0, 0, 0};	// Store timings for first white gap, black segment, second white gap, black:white ratio.
unsigned char intersectionDir;					// Direction that should be taken at the next intersection.
unsigned char trackingState, lastTrackingState;	// Flag when over a gap/white area, previous flag when over a gap/white area.


unsigned int timePeriod, i;



/****************************************************************************************
 * Main programme that the ATMega8 runs
 *
 * Initialises global variables, starts features required, enables global interrupts,
 * then continuously calculates direction of the robot's motors.
 ****************************************************************************************/
int main(void) {
	cli();					// Disable interrupts

	initialise();			// Initialise global variables
	startUp();				// Start up the dataOut, dipSwtich, timer, pwm and light sensing functionality

	sei();					// Enable interrupts
	
	//int temp = 0;
	for(;;) {
		// Take the most recent sensor values
		sensorSnapShot = sensor;
		gapSense(sensorSnapShot);
		movement(sensorSnapShot);
	/*	if (timerHasTimeOut()) {
			(temp = ~temp) ? (PORTB &= ~(1 << PINB7)) : (PORTB |= (1 << PINB7));
			timerReset();
		}*/
	}
	
	return 0;
}


/* Initialise global variables used
 */
inline void initialise() {
	counter = 0x0;					// Set sensor count to zero
	sensor = 0b0010;				// Assume middle sensor is on
	trackingState = TRACK;			// Assume robot is on a black line
	lastTrackingState = TRACK;		// Assume robot is on a black line
	motorDir = MIDDLE;				// Set robot to move in a straight line
	ADCroutine = &dipSwitchADC;		// Run the routine to determine dip switch positions
	
	timerReset();					// Set timerTicks and isTiming to zero
	intersectionDir = 0;			// Set direction of first intersection to null
	timePeriod = 0;
	i = 0;

	// Set H-Bridge PWMs off
	OCR1A = OFF;		// Right motor
	OCR1B = OFF;		// Left motor
	OCR2 = OFF;			// Reverse for both motors
}

/* Sets values in Analog to Digital Converter, Timer Interrupt Masks, Timer0,
 * Timer1 and Timer2 registers to turn required features on.
 */
inline void startUp() {
	/* dipSwitch()
	 * Prescale system clock by 64
	 * Select port Pin C3 as ADC source
	 ************************************/
	ADCSRA = (1 << ADEN) | (1 << ADSC) | (0 << ADFR) | (1 << ADIE) | (0b110);
	ADMUX = (1 << ADLAR) | (0b0011);
	

	/* dataOut()
	 * Set data out port on Pin B0 as output
	 * Also set Pin B6 & 7 as debugging outputs
	 ************************************/
	DDRB |= (1 << DDB0) | (1 << DDB6) | (1 << DDB7);


	/* sensors()
	 * Uses Port C0, C1 and C2 pins to handle the sensing of black/white levels.
	 * Pin D5 used to turn light LED on.
	 * (uses Timer0 for sensing period).
	 ************************************/
	DDRC |= (1 << DDC2) | (1 << DDC1) | (1 << DDC0);		// The 3 sensors.
	DDRD |= (1 << DDD5);									// Light on


	/* timer()
	 * Timer0.  No prescaling
	 * Enable overflow
	 ************************************/
	TCCR0 |= (1 << CS00);
	TIMSK |= (1 << TOIE0);


	/* pwm()
	 *
	 * Uses the 16 Bit Timer1 for fast 8-bit PWM
	 *  -> Going forward independently for HBridge2_2 and HBridge1_1
	 *
	 * Uses the 8 Bit Timer2 for fast 8-bit PWM
	 *  -> Going reverse for HBridge2_1 and HBridge1_2
	 ************************************/
	//** HBridge2_2 and HBridge1_1
	//	Timer1
	// 	Set Pin B1 & B2 as outputs.
	// 	Set to use Fast PWM 8-bit.  Prescale 8.
	DDRB |=  (1 << DDB2) | (1 << DDB1);
	TCCR1A = (1 << COM1A1) | (1 << COM1A0) |(1 << COM1B1) | (1 << COM1B0) | (1 << WGM10);	// Compare timer1 to register A & B
	TCCR1B = (1 << WGM12) | (1 << CS11);


	//** HBridge2_1 and HBridge1_2
	//	Timer2
	//	Set Pin B4 and Pin D6 as outputs.
	//	Set to use Fast PWM
	//	Compare timer2 to register OCR2.  Prescale 8.
	//	Enable interrupts for port replication
	//DDRB |= (1 << DDB4);
	//DDRD |= (1 << DDD6);
	//TCCR2 = (1 << WGM20) | (1 << COM21) | (1 << COM20) | (1 << WGM21) | (1 << CS21);
	//TIMSK |= (1 << OCIE2) | (1 << TOIE2);
	
	
	
	/* timerStart()
	 * Start the gap timer.
	 ************************************/
	// timerStart();
	
	/* Let micro-controller stabilise before enabling interrupts
	 * and providing power to motor.
	 ************************************/
	for(delay = 100000; delay; delay--);
}


/*#############################################################################
############################### Interrupts ####################################
###############################################################################*/


/* Timer0 overflow interrupt service routine.
 *
 * Timer0 frequency = 1 MHz
 * PWM frequency = 4.10 KHz
 * Sensing frequency = 370 Hz
 *
 *
 *      Tristate
 * |------|-|---------------------|
 * Reverse     Discharge period
 *   bias                         ^
 *                                | End of period.
 *                                | Take measurement.
 *
 * PIN C2, C1, C0 = Right, Middle, Left sensors
 *
 * dataOut pin = Pin B0
 */
ISR(TIMER0_OVF_vect) {
	// Periodically reverse biasing and discharging LED light sensors
	switch(counter++) {
		// Reverse bias
		case(0x01):		DDRC |= (1 << DDC2) | (1 << DDC1) | (1 << DDC0);		// Set Pin C2, C1, C0 directions to output
						PORTC |= (1 << PINC2) | (1 << PINC1) | (1 << PINC0); 	// and set pins to high
						break;

		// Tristate sensors
		case(0x50):		DDRC ^= (1 << DDC2) | (1 << DDC1) | (1 << DDC0);		// Set Pin C2, C1 & C0 directions to input
						PORTC ^= (1 << PINC2) | (1 << PINC1) | (1 << PINC0);	// and set pins to low
						break;

		// End of discharge period
		case(0x58):		sensor = PINC & 0b0111;		// PIN C2, C1, C0 == Right, Middle, Left
						counter = 0x00;				// Reset counter
						break;
					
		default:		break;
	}


	// Increment clock when timing is enabled
	timerTicks += isTiming;

	if (i++ < 10000 && mode == 3) {
		switch (timePeriod++) {
			case(1):	PORTB &= ~(1 << PINB0);
						break;
						
			case(25):	PORTB |= (1 << PINB0);
						break;
						
			case(50):	timePeriod = 0;
						break;
						
			default:	break;
		}	
	} else if (i < 20000 && mode == 3) {
		switch (timePeriod++) {
			case(1):	PORTB &= ~(1 << PINB0);
						break;
						
			case(20):	PORTB |= (1 << PINB0);
						break;
						
			case(40):	timePeriod = 0;
						break;
						
			default:	break;
		}		
	} else {
		i = 0;
		timePeriod = 0;
	}
}


/* Timer2 compare interrupt service routine
 * Used for setting Pins high to both H-Bridges for reversing; Hight part
 * of PWM signal controlled by OCR2 value set in movement()
 *
 * Left					Right
 * PinB4				PinD6
 */
ISR(TIMER2_COMP_vect) {
/*	switch (OCR2 == 0x00 || OCR2 == 0xFF) {
		case(1):	break;						// Boundary condition.  Let Timer2_Ovf ISR handle it

		default:	PORTB |= (1 << PINB4);		// Put Pin B4 on
					PORTD |= (1 << PIND6);		// Put Pin D6 on
					break;
	}*/
}


/* Timer2 overflow interrupt service routine
 * Used for setting Pins low to both H-Bridges for reversing; Low part
 * of PWM signal controlled by OCR2 value set in movement()
 * OCR2 value set in movement()
 *
 * Left					Right
 * PinB4				PinD6
 */
ISR(TIMER2_OVF_vect) {	
/*	switch (OCR2) {
		// 100% duty cycle
		case(0x00):	PORTB |= (1 << PINB4);		// Keep Pin B4 on
					PORTD |= (1 << PIND6);		// Keep Pin D6 on
					break;

		// 0% duty cycle
		case(0xFF):	PORTB &= ~(1 << PINB4);	// Turn Pin B4 off
					PORTD &= ~(1 << PIND6);	// Turn Pin D6 off
					break;

		default:	PORTB ^= (1 << PINB4);		// Toggle Pin B4
					PORTD ^= (1 << PIND6);		// Toggle Pin D6
					break;
	}*/
}


/* Analog Digital Converter interrupt service routine
 * First used to determine dip switch position.
 * Then used to determine strength of IPT source.
 * 
 * Pointer to the appropriate function is used.
 */
ISR(ADC_vect) {
	ADCroutine();
}


/*#############################################################################
############################## Main functions #################################
###############################################################################*/


/* Dip Switch Analog Digital Converter interrupt function
 * Reads in voltage set by 4 way DIP switch and interprets it
 * as a mode for the robot to operate in.
 * 
 * Some global variables and output pins set depending on mode.
 *
 * Free running disabled.  Interrupt run once.
 */
void dipSwitchADC() {
	switch(ADCH) {
		// At ground 0V
		case(0x00):		mode = 4;
						ADCroutine = &iptADC;
						ADCSRA |= (1 << ADSC) | (1 << ADFR);		//   _ _ _ 4	IPT sensing
						break;
		
		// At reference 5V
		case(0xFF):		mode = 1;
						PORTB |= (1 << PINB0);						//   1 _ _ _	Not used
						break;
		
		// In between
		default:		if (0x50 < ADCH && ADCH < 0x60) {			// 1.667V is between 1.58V and 1.88V
							mode = 3;								//   _ _ 3 _	Efficiency mode
							PORTD |= (1 << PIND5);					//   Turn lights on
						} else if (0xA0 < ADCH && ADCH < 0xB0) {	// 3.333V is between 3.14V and 3.45V
							mode = 2;
							soundGenerator();						//   _ 2 _ _  Sound generation
						} else {									// Otherwise
							mode = 0;								//   _ _ _ _  Not used
						}
						break;
	}
}


/* IPT Analog Digital Converter interrupt function
 * Reads in voltage from the two IPT sensors
 * Determines the IPT field strength and lines
 *
 * Free running disabled after IPT sensing is not required.
 *  - Done in the logic determining IPT action.
 */
void iptADC() {
	// Alternate between the two inputs of the IPT sensors
	if ((ADMUX & 0b0111) == 0b0100) {
		ADCH ? (PORTB |= (1 << PINB6)) : (PORTB &= ~(1 << PINB6));
		ADMUX = (1 << ADLAR) | (0b0101);		// Note, assignment statement (no OR-ing)
	} else {
		ADMUX = (1 << ADLAR) | (0b0100);
		ADCH ? (PORTB |= (1 << PINB7)) : (PORTB &= ~(1 << PINB7));
	}
}


/* Function determines the amount of movement
 * for each side of the robot and then applies result
 * to each H-Bridge.
 *
 * Next state is determined by using sensor values as
 * an array index into a look up table.  The entry returned
 * is the next state that the motor should be in.
 * This implementation is reactive and disregards the previous
 * state.  Mealy FSM.  However, the momentum of the robot
 * provides a memory effect.
 *
 * Lower compare numbers = higher duty cycle
 *				
 * Left					Right
 * OCR1B				OCR1A
 * PinB2				PinB1
 *
 * Reverse = OCR2
 */
void movement(unsigned char sensorSnapShot) {
	(mode == 3) ? (motorDir = nextMotorDir[sensorSnapShot]) : (motorDir = IDLE);
	
	// Reverse drive usually unused
	OCR2 = OFF;
	
	// Applying next state to the H-Bridges (controls motor speed)
	switch(motorDir) {
		// #### Normal speed ####								Previous implementation with memory effect:
		// Turn left.  Right wheel turns more
		case(LEFT):		OCR1A = MEDIUM;							// (OCR1A == 0x30) ? 0 : (OCR1A--);
						OCR1B = OFF;							// (OCR1B < 0x70) ? (OCR1B++) : (OCR1B--);
						break;
	
		// Turn right.  Left wheel turns more
		case(RIGHT):	OCR1A = OFF;							// (OCR1A < 0x70) ? (OCR1A++) : (OCR1A--);
						OCR1B = MEDIUM;							// (OCR1B == 0x30) ? 0 : (OCR1B--);
						break;

		// Turn on.  Both wheels turn
		case(MIDDLE):	OCR1A = MEDIUM;							// (OCR1A == 0x30) ? 0 : (OCR1A--);
						OCR1B = MEDIUM;							// (OCR1B == 0x30) ? 0 : (OCR1B--);
						break;
				
		
		// #### Medium speed ####
		// Turn left.  Right wheel turns more
		case(LEFT_M):	OCR1A = MEDIUM;
						OCR1B = SLOW;
						break;
	
		// Turn right.  Left wheel turns more
		case(RIGHT_M):	OCR1A = SLOW;
						OCR1B = MEDIUM;
						break;
		
		
		// Turn off.  Both wheels off
		case(IDLE):		OCR1A = OFF;							// (OCR1A == 0xFF) ? 0 : (OCR1A++);
						OCR1B = OFF;							// (OCR1B == 0xFF) ? 0 : (OCR1B++);
						break;
		
		/* Put motor in reversed state
		case(REVERSE):	OCR1A = OFF;
						OCR1B = OFF;
						OCR2 = MEDIUM;
						break;
		
		// Apply braking
		case(BRAKE):	OCR1A = OFF;							//
						OCR1B = OFF;							// Turn forward signals off

						OCR2 = MEDIUM;							// Apply medium duty cycle reverse signal
						for(unsigned int delay = 50000; delay; delay--);

						OCR2 = OFF;								// Turn reverse signal off.  Robot should have stopped
						motorDir = IDLE;						// Set motor to idle state.
						break;*/
		
		
		case(INTERSECTION):
						OCR1A = MEDIUM;
						OCR1B = MEDIUM;
						switch(marker[3]) {
							// Turn left
							case(LEFT_TURN):	for (unsigned int delay = 16000; delay; delay--);
												OCR1A = FAST;
												OCR1B = OFF;
												
												for (unsigned long int delay = 32000; delay; delay--);
												break;
										
							// Turn right
							case(RIGHT_TURN):	for (unsigned int delay = 20000; delay; delay--);
												OCR1A = OFF;
												OCR1B = FAST;
												
												for (unsigned long int delay = 40000; delay; delay--);
												break;
											
							// Go straight ahead
							case(STRAIGHT):		for (unsigned int delay = 5000; delay; delay--);
												break;
											
							// Turn into IPT station
							case(IPT):			break;
							
							// Stop the robot
							case(END):			OCR1A = OFF;
												OCR1B = OFF;
												PORTD &= ~(1 << PIND5);
												for ( ; ; );	// Infinite loop
												break;
											
							default:			break;
						}
						marker[3] = 0;		// Reset intersection state to zero (serviced).
						break;
		
		default:		break;
	}
}


/* Timing functionality to hide the complexity of managing timer variables
 * Inlined to reduce function call overhead.
 * Could be replaced by #defines
 *
 * Reset, stop, start, timeout functions
 */
inline void timerReset() { timerTicks = 0; isTiming = 1;}
inline void timerStart() { isTiming = 1;}
inline void timerStop() { isTiming = 0;}
inline unsigned char timerHasTimeOut() { return (timerTicks > TIMEOUT);}


/*#############################################################################
################################ Gap Handlers #################################
###############################################################################*/


/* Function detects instances of line encodings
 *
 * Starts and stops the timer at appropriate transitions.
 *
 * Stores time in marker[] array.
 *   marker[0] = First gap time
 *   marker[1] = Line segment time
 *   marker[2] = Second gap time
 */
void gapSense(unsigned char sensorSnapShot) {
	(sensor & 0b0010) ? (trackingState = LINE) : (trackingState = GAP);		// True when sensors have not discharged

	// When timer runs for too long, assume robot is still on the line and reset gap detection
	if (timerHasTimeOut()) {
		lastTrackingState = TRACK;
		timerReset();		// Give timer a clean slate
		timerStop();		//
		marker[0] = 1;		//
		marker[1] = 100;	//
		marker[2] = 1;		// Set bogus values for line code timings
		marker[3] = 0;		//
	}

	switch(trackingState) {
		case(GAP) :
			switch(lastTrackingState) {
				// From line (track) to first gap
				// Reset and start the timer.
				// Robot has now entered line code.
				case(TRACK):	timerReset();
								lastTrackingState = FIRSTGAP;
								break;

				// From line segment to second gap
				// Stop timer and take time
				// Restart timer
				// Start timer
				case(LINESEGMENT):	marker[1] = timerTicks;
									timerReset();
									lastTrackingState = SECONDGAP;
									break;
					
				default:		break;
			}
			break;

		case(LINE):
			switch(lastTrackingState) {
				// From first gap to line segment
				// Stop the timer and take time
				// Restart timer
				// Start timer
				case(FIRSTGAP):	marker[0] = timerTicks;
								timerReset();
								lastTrackingState = LINESEGMENT;
								break;
				
				// From second gap to line (track)
				// Stop timer and take time
				// Reset timer
				// Robot has now gone through the line code.
				case(SECONDGAP):	marker[2] = timerTicks;
									timerReset();
									timerStop();
									gapInterpretation();				// Interpret code
									lastTrackingState = TRACK;
									break;
								
				default:		break;
			}
			break;
			
		default:	break;
	}
}


/* Function determines the direction encoded in the line
 * after traversal
 *
 * marker[0] = First gap time
 * marker[1] = Line segment time
 * marker[2] = Second gap time
 * marker[3] = Black:White ratio
 */
void gapInterpretation(void) {
	unsigned long int temp = (marker[1] << 2)*5;
	marker[3] = temp / (marker[0] + marker[2]);


	if (5 < marker[3] && marker[3] < 15) { marker[3] = LEFT_TURN;}			// 1 cm
	else if (15 < marker[3] && marker[3] < 25) { marker[3] = RIGHT_TURN;}		// 2 cm
	else if (25 < marker[3] && marker[3] < 35) { marker[3] = STRAIGHT;}		// 3 cm
	else if (35 < marker[3] && marker[3] < 45) { marker[3] = IPT;}			// 4 cm
	else if (45 < marker[3] && marker[3] < 55) { marker[3] = END;}			// 5 cm		
	else { marker[3] = 0;}
}



/* Function that generates tones for musical play back
 *
 * Tones implemented as for-loops controlling
 * pitch and duration.
 */
void soundGenerator(void) {
	for ( ; ;) {
	//	playNote(WHOLE_NOTE, C4);
	//	playKookaburra();
	//	playCanon();
	//	playRest(WHOLE_REST);
		playAir();
		playRest(WHOLE_REST);
	}
}


void playNote(unsigned long int beats, unsigned int note) {
	for (unsigned int period = beats/note; period; period--) {
		PORTB |= (1 << PINB0);
		for (unsigned int note1 = note; note1; note1--);
			
		PORTB &= ~(1 << PINB0);
		for (unsigned int note2 = note; note2; note2--);
	}
}

void playRest(unsigned long int beats) {
	PORTB &= ~(1 << PINB0);
	for (unsigned long int period = beats; period; period--);
}


void playScale(void) {
/*	playNote(WHOLE_NOTE, C5);
	playNote(WHOLE_NOTE, C5_SHARP);
	playNote(WHOLE_NOTE, D5);
	playNote(WHOLE_NOTE, D5_SHARP);
	playNote(WHOLE_NOTE, E5);
	playNote(WHOLE_NOTE, F5);
	playNote(WHOLE_NOTE, F5_SHARP);
	playNote(WHOLE_NOTE, G5);
	playNote(WHOLE_NOTE, G5_SHARP);
	playNote(WHOLE_NOTE, A5);
	playNote(WHOLE_NOTE, A5_SHARP);
	playNote(WHOLE_NOTE, B5);
	
	playNote(WHOLE_NOTE, C4);
	playNote(WHOLE_NOTE, C4_SHARP);
	playNote(WHOLE_NOTE, D4);
	playNote(WHOLE_NOTE, D4_SHARP);
	playNote(WHOLE_NOTE, E4);
	playNote(WHOLE_NOTE, F4);
	playNote(WHOLE_NOTE, F4_SHARP);
	playNote(WHOLE_NOTE, G4);
	playNote(WHOLE_NOTE, G4_SHARP);
	playNote(WHOLE_NOTE, A4);
	playNote(WHOLE_NOTE, A4_SHARP);
	playNote(WHOLE_NOTE, B4);
	
	playNote(WHOLE_NOTE, C3);
	playNote(WHOLE_NOTE, C3_SHARP);
	playNote(WHOLE_NOTE, D3);
	playNote(WHOLE_NOTE, D3_SHARP);
	playNote(WHOLE_NOTE, E3);
	playNote(WHOLE_NOTE, F3);
	playNote(WHOLE_NOTE, F3_SHARP);
	playNote(WHOLE_NOTE, G3);
	playNote(WHOLE_NOTE, G3_SHARP);
	playNote(WHOLE_NOTE, A3);
	playNote(WHOLE_NOTE, A3_SHARP);
	playNote(WHOLE_NOTE, B3);*/
}

void playKookaburra(void) {
/*	playNote(EIGHTH_NOTE, D3);
	playNote(EIGHTH_NOTE, D3);
	playNote(EIGHTH_NOTE, D3);
	playNote(EIGHTH_NOTE, D3);
	
	playNote(QUARTER_NOTE, E3);
	playNote(EIGHTH_NOTE, E3);
	playNote(EIGHTH_NOTE, E3);
	
	//---------
	playNote(QUARTER_NOTE, D3);
	playNote(QUARTER_NOTE, B4);
	playNote(QUARTER_NOTE, D3);
	playNote(QUARTER_NOTE, B4);
	
	//---------
	playNote(EIGHTH_NOTE, B4);
	playNote(EIGHTH_NOTE, B4);
	playNote(EIGHTH_NOTE, B4);
	playNote(EIGHTH_NOTE, B4);
	
	playNote(QUARTER_NOTE, C3);
	playNote(EIGHTH_NOTE, C3);
	playNote(EIGHTH_NOTE, C3);
	
	//---------
	playNote(QUARTER_NOTE, B4);
	playNote(QUARTER_NOTE, G4);
	playNote(QUARTER_NOTE, B4);
	playNote(QUARTER_NOTE, G4);
	
	//---------
	playNote(HALF_NOTE, G3);
	playNote(EIGHTH_NOTE, E3);
	playNote(EIGHTH_NOTE, F3_SHARP);
	playNote(EIGHTH_NOTE, G3);
	playNote(EIGHTH_NOTE, E3);
	
	//---------
	playNote(HALF_NOTE, D3);
	playNote(EIGHTH_NOTE, D3);
	playNote(EIGHTH_NOTE, E3);
	playNote(EIGHTH_NOTE, D3);
	playNote(EIGHTH_NOTE, C3);
	
	//---------
	playNote(QUARTER_NOTE, B4);
	playNote(QUARTER_NOTE, G4);
	playNote(QUARTER_NOTE, G4);
	playNote(QUARTER_NOTE, G4);
	
	playNote(WHOLE_NOTE, G4);*/
}

void playCanon(void) {
/*	//######### Line 1 #########
	playNote(HALF_NOTE, E4);
	playNote(HALF_NOTE, D4);
	playNote(HALF_NOTE, C4);
	playNote(HALF_NOTE, B5);

	playNote(HALF_NOTE, A5);
	playNote(HALF_NOTE, G5);
	playNote(HALF_NOTE, A5);
	playNote(HALF_NOTE, B5);
	
	//######### Line 2 #########
	playNote(HALF_NOTE, E3);
	playNote(HALF_NOTE, D3);
	playNote(HALF_NOTE, C3);
	playNote(HALF_NOTE, B4);

	playNote(HALF_NOTE, A4);
	playNote(HALF_NOTE, G4);
	playNote(HALF_NOTE, A4);
	playNote(HALF_NOTE, B4);

	//######### Line 3 #########
	playNote(QUARTER_NOTE, C3);
	playNote(QUARTER_NOTE, C3);
	playNote(QUARTER_NOTE, D3);
	playNote(QUARTER_NOTE, B4);

	playNote(QUARTER_NOTE, C3);
	playNote(QUARTER_NOTE, E3);
	playNote(QUARTER_NOTE, G3);
	playNote(QUARTER_NOTE, G4);

	playNote(QUARTER_NOTE, A4);
	playNote(QUARTER_NOTE, F4);
	playNote(QUARTER_NOTE, E4);
	playNote(QUARTER_NOTE, G4);

	playNote(QUARTER_NOTE, F4);
	playNote(QUARTER_NOTE, C3);
	playNote(QUARTER_NOTE, B4);
	playNote(QUARTER_NOTE, G4);

	playNote(QUARTER_NOTE, C3);
	playNote(EIGHTH_NOTE, E3);
	playNote(EIGHTH_NOTE, G3);
	playNote(EIGHTH_NOTE, G3);
	playNote(EIGHTH_NOTE, A3);
	playNote(EIGHTH_NOTE, G3);
	playNote(EIGHTH_NOTE, F3);

	//######### Line 4 #########
	playNote(EIGHTH_NOTE + QUARTER_NOTE, E3);
	playNote(EIGHTH_NOTE, E3);
	playNote(EIGHTH_NOTE, E3);
	playNote(EIGHTH_NOTE, F3);
	playNote(EIGHTH_NOTE, E3);
	playNote(EIGHTH_NOTE, D3);

	playNote(EIGHTH_NOTE, C3);
	playNote(EIGHTH_NOTE, A4_SHARP);
	playNote(EIGHTH_NOTE, A4);
	playNote(EIGHTH_NOTE, A4_SHARP);
	playNote(QUARTER_NOTE, G4);
	playNote(QUARTER_NOTE, E4);

	playNote(QUARTER_NOTE, C4);
	playNote(EIGHTH_NOTE, F4);
	playNote(EIGHTH_NOTE, E4);
	playNote(QUARTER_NOTE, D4);
	playNote(EIGHTH_NOTE, G4);
	playNote(EIGHTH_NOTE, F4);

	playRest(QUARTER_REST);
	playNote(QUARTER_NOTE, C3);
	playNote(QUARTER_NOTE, D3);
	playNote(QUARTER_NOTE, B4);

	//######### Line 5 #########
	playNote(QUARTER_NOTE, C3);
	playNote(QUARTER_NOTE, E4);
	playNote(QUARTER_NOTE + EIGHTH_NOTE, G4);
	playNote(EIGHTH_NOTE, A4);

	playNote(QUARTER_NOTE, F4);
	playNote(QUARTER_NOTE, C4);
	playNote(QUARTER_NOTE, E4);
	playNote(QUARTER_NOTE, G4);

	playNote(QUARTER_NOTE, F4);
	playNote(QUARTER_NOTE, E4);
	playNote(QUARTER_NOTE, D4);
	playNote(QUARTER_NOTE, G4);
	playNote(WHOLE_NOTE, C4);*/
}

void playAir(void) {
	for (unsigned short int i = 2; i; i--) {
		//######### Line 1 #########
		playNote(WHOLE_NOTE + WHOLE_NOTE + QUARTER_NOTE, C3_SHARP);
		playNote(EIGHTH_NOTE, F3_SHARP);
		playNote(EIGHTH_NOTE, D3);
		playNote(EIGHTH_NOTE, B4);
		playNote(EIGHTH_NOTE, A4);
		playNote(EIGHTH_NOTE, G4_SHARP);
		playNote(EIGHTH_NOTE, A4);

		playNote(HALF_NOTE, G4_SHARP);
		playNote(EIGHTH_NOTE, F4_SHARP);
		playNote(QUARTER_NOTE + EIGHTH_NOTE, E4);

		//######### Line 2 #########
		playNote(WHOLE_NOTE + EIGHTH_NOTE, E3);
		playNote(EIGHTH_NOTE, C3_SHARP);
		playNote(EIGHTH_NOTE, F4_SHARP);
		playNote(EIGHTH_NOTE, F4_SHARP);

		playNote(EIGHTH_NOTE, B4);
		playNote(EIGHTH_NOTE, A4_SHARP);
		playNote(EIGHTH_NOTE, E3);
		playNote(EIGHTH_NOTE, D3);

		playNote(WHOLE_NOTE + EIGHTH_NOTE, D3);

		//######### Line 3 #########
		playNote(EIGHTH_NOTE, B4);
		playNote(EIGHTH_NOTE, F4_SHARP);
		playNote(EIGHTH_NOTE, E4);
		playNote(EIGHTH_NOTE, A4);
		playNote(EIGHTH_NOTE, G4_SHARP);
		playNote(EIGHTH_NOTE, D3);
		playNote(EIGHTH_NOTE, C3_SHARP);

		playNote(HALF_NOTE + QUARTER_NOTE, C3_SHARP);
		playNote(EIGHTH_NOTE, D3_SHARP);
		playNote(EIGHTH_NOTE, E3);

		playNote(QUARTER_NOTE, A4);
		playNote(SIXTEENTH_NOTE, A4);
		playNote(SIXTEENTH_NOTE, B4);
		playNote(EIGHTH_NOTE, C3_SHARP);
		playNote(EIGHTH_NOTE, C3_SHARP);
		playNote(EIGHTH_NOTE, B4);
		playNote(EIGHTH_NOTE, B4);
		playNote(EIGHTH_NOTE, A4);

		//######### Line 4 #########
		playNote(EIGHTH_NOTE, G4_SHARP);
		playNote(EIGHTH_NOTE, F4_SHARP);
		playNote(SIXTEENTH_NOTE, F4_SHARP);
		playNote(SIXTEENTH_NOTE, G4_SHARP);
		playNote(EIGHTH_NOTE, A4);
		playNote(QUARTER_NOTE, A4);
		playNote(EIGHTH_NOTE, G4_SHARP);
		playNote(EIGHTH_NOTE, F4_SHARP);

		playNote(WHOLE_NOTE, E4);
	}
	playNote(WHOLE_NOTE, E4);


	for (unsigned short int i = 2; i; i--) {
		playNote(HALF_NOTE + QUARTER_NOTE, G4_SHARP);
		playNote(SIXTEENTH_NOTE, A4);
		playNote(SIXTEENTH_NOTE, G4_SHARP);
		playNote(SIXTEENTH_NOTE, F4_SHARP);
		playNote(SIXTEENTH_NOTE, G4_SHARP);
		playNote(QUARTER_NOTE, E4);

		//######### Line 5 #########
		playNote(HALF_NOTE + QUARTER_NOTE, E3);
		playNote(QUARTER_NOTE, F4_SHARP);
		playNote(QUARTER_NOTE, F4_SHARP);
		playNote(QUARTER_NOTE, F3_SHARP);
		playNote(EIGHTH_NOTE, F3_SHARP);
		playNote(EIGHTH_NOTE, E3);
		playNote(EIGHTH_NOTE, D3);
		playNote(EIGHTH_NOTE, C3_SHARP);

		playNote(HALF_NOTE + SIXTEENTH_NOTE, D3);
		playNote(SIXTEENTH_NOTE, C3_SHARP);
		playNote(SIXTEENTH_NOTE, B4);
		playNote(SIXTEENTH_NOTE, A4);
		playNote(EIGHTH_NOTE, G4_SHARP);
		playNote(EIGHTH_NOTE, F4_SHARP);

		//######### Line 6 #########
		playNote(EIGHTH_NOTE, E4);
		playNote(EIGHTH_NOTE, F4_SHARP);
		playNote(QUARTER_NOTE, G4_SHARP);
		playNote(EIGHTH_NOTE, G4_SHARP);
		playNote(EIGHTH_NOTE, A4);
		playNote(QUARTER_NOTE, B4);

		playNote(EIGHTH_NOTE, B4);
		playNote(EIGHTH_NOTE, C3_SHARP);
		playNote(HALF_NOTE, D3);
		playNote(QUARTER_NOTE, C3_SHARP);

		playNote(EIGHTH_NOTE, B4);
		playNote(EIGHTH_NOTE, A4);
		playNote(EIGHTH_NOTE, G4_SHARP);
		playNote(EIGHTH_NOTE, F4_SHARP);

		playNote(EIGHTH_NOTE, G4_SHARP);
		playNote(SIXTEENTH_NOTE, A4);
		playNote(SIXTEENTH_NOTE, B4);
		playNote(QUARTER_NOTE, A4);

		//######### Line 7 #########
		playNote(WHOLE_NOTE, F4_SHARP);
		playNote(HALF_NOTE + EIGHTH_NOTE, A4);
		playNote(EIGHTH_NOTE, C3_SHARP);
		playNote(EIGHTH_NOTE, B4);
		playNote(EIGHTH_NOTE, A4);

		playNote(HALF_NOTE + QUARTER_NOTE, F3_SHARP);
		playNote(EIGHTH_NOTE, E3);
		playNote(EIGHTH_NOTE, D3_SHARP);

		playNote(SIXTEENTH_NOTE, C3_SHARP);
		playNote(SIXTEENTH_NOTE, B4);
		playNote(EIGHTH_NOTE, E3);
		playNote(QUARTER_NOTE, E4);
		playNote(QUARTER_NOTE + EIGHTH_NOTE, F4_SHARP);
		playNote(SIXTEENTH_NOTE, G4_SHARP);
		playNote(SIXTEENTH_NOTE, A4);

		//######### Line 8 #########
		playNote(QUARTER_NOTE + EIGHTH_NOTE, G4_SHARP);
		playNote(EIGHTH_NOTE, F4_SHARP);
		playNote(HALF_NOTE, E4);

		playNote(HALF_NOTE + QUARTER_NOTE, A4);
		playNote(EIGHTH_NOTE, C3_SHARP);
		playNote(EIGHTH_NOTE, B4);

		playNote(HALF_NOTE + QUARTER_NOTE, B4);
		playNote(EIGHTH_NOTE, D3);
		playNote(EIGHTH_NOTE, C3_SHARP);

		//######### Line 9 #########
		playNote(HALF_NOTE + QUARTER_NOTE, C3_SHARP);
		playNote(QUARTER_NOTE, E3);
		playNote(QUARTER_NOTE, D3);

		playNote(WHOLE_NOTE, D3);

		playNote(HALF_NOTE + EIGHTH_NOTE, E4);
		playNote(EIGHTH_NOTE, G4_SHARP);
		playNote(EIGHTH_NOTE, B4);
		playNote(EIGHTH_NOTE, D3);

		//######### Line 10 #########
		playNote(EIGHTH_NOTE, D3);
		playNote(EIGHTH_NOTE, B4);
		playNote(HALF_NOTE + EIGHTH_NOTE, C3_SHARP);
		playNote(SIXTEENTH_NOTE, D3);
		playNote(SIXTEENTH_NOTE, E3);

		playNote(HALF_NOTE + EIGHTH_NOTE, A4);
		playNote(EIGHTH_NOTE, C3_SHARP);
		playNote(EIGHTH_NOTE, E3);
		playNote(EIGHTH_NOTE, F3_SHARP);

		playNote(HALF_NOTE + QUARTER_NOTE, F3_SHARP);
		playNote(QUARTER_NOTE, A4);

		//######### Line 11 #########
		playNote(EIGHTH_NOTE, G4_SHARP);
		playNote(EIGHTH_NOTE, B4);
		playNote(HALF_NOTE, D3);
		playNote(QUARTER_NOTE, F4_SHARP);

		playNote(QUARTER_NOTE, E4);
		playNote(EIGHTH_NOTE, B4);
		playNote(SIXTEENTH_NOTE, C3_SHARP);
		playNote(SIXTEENTH_NOTE + EIGHTH_NOTE, D3);
		playNote(QUARTER_NOTE, C3_SHARP);
		playNote(EIGHTH_NOTE, B4);

		playNote(SIXTEENTH_NOTE, A4);
		playNote(SIXTEENTH_NOTE, G4_SHARP);
		playNote(QUARTER_NOTE, F4_SHARP);
		playNote(EIGHTH_NOTE, G4_SHARP);
		playNote(QUARTER_NOTE, A4);
		playNote(EIGHTH_NOTE, G4_SHARP);
		playNote(EIGHTH_NOTE, A4);

		playNote(WHOLE_NOTE, A4);
	}

}

